// Load in dependencies
var extend = require('obj-extend');
var request = require('request');
var sax = require('sax');

// Define our CSS validator
function validateCss(options, cb) {
  // TODO: This can be refactored into a class if it becomes unwieldy in size (e.g. error parsing)
  // If options is a string, upcast it to an object
  if (typeof options === 'string') {
    var css = options;
    options = {text: css};
  }

  // Grab the URL we are going to POST to
  var w3cUrl = options.w3cUrl || validateCss.w3cUrl;

  // Open the request
  // http://jigsaw.w3.org/css-validator/api.html
  // TODO: It would be nice to return a stream if no cb was called (but one that emits 'validation-error' events)
  request({
    url: w3cUrl,
    qs: extend({}, options, {output: 'soap12'})
  }, function (err, res, body) {
    // If there was an error, callback
    if (err) {
      return cb(err);
    }

    // Parse the XML
    // TODO: It would be awesome if we can stream in data to validateCss
    // DEV: We aren't using a SOAP node module because all of them are overkill

    // Create placeholder for data
    // TODO: We should emit this as a stream for the non-cb mode
    var validationErrors = [];
    var validationWarnings = [];
    var result = {validity: false, errors: validationErrors, warnings: validationWarnings};

    // Parse the XML
    var xmlParser = sax.createStream();

    // Save validity and any error info
    var validationErr;
    xmlParser.on('text', function (text) {
      var node = this._parser.tag;
      var nameNS = node.name.toLowerCase();
      if (nameNS === 'm:validity') {
        result.validity = text === 'true';
      } else if (validationErr) {
        var name = nameNS.replace('m:', '');
        validationErr[name] = text;
      }
    });

    xmlParser.on('opentag', function (node) {
      switch (node.name.toLowerCase()) {
        case 'm:error':
        case 'm:warning':
          validationErr = {};
          break;
      }
    });
    xmlParser.on('closetag', function (name) {
      var node = this._parser.tag;
      switch (node.name.toLowerCase()) {
        case 'm:error':
          validationErrors.push(validationErr);
          validationErr = null;
          break;
        case 'm:warning':
          validationWarnings.push(validationErr);
          validationErr = null;
          break;
      }
    });

    xmlParser.on('end', function () {
      cb(null, result);
    });

    xmlParser.write(body);
    xmlParser.end();
  });
}

// Set up reference for default end point
// http://jigsaw.w3.org/css-validator/manual.html#api
validateCss.w3cUrl = 'http://jigsaw.w3.org/css-validator/validator';

// Export validateCss
module.exports = validateCss;